# from plone.app.vocabularies.catalog import SearchableTextSourceBinder
from bs4 import BeautifulSoup
from plone.app.portlets.portlets import base
from plone.portlets.interfaces import IPortletDataProvider
from zope import schema
from zope.interface import implementer

from Products.EasyNewsletter import EasyNewsletterMessageFactory as _  # noqa
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

VALID_TAGS = [
    "strong",
    "b",
    "em",
    "p",
    "a",
    "ul",
    "ol",
    "li",
    "br",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
]


def sanitize_html(value):

    soup = BeautifulSoup(value)

    for tag in soup.findAll(True):
        if tag.name not in VALID_TAGS:
            tag.hidden = True

    return soup.renderContents()


class INewsletterSubscriberPortlet(IPortletDataProvider):
    """ """

    portlet_title = schema.TextLine(
        title=_("Title for the portlet."), default="Newsletter", required=True
    )

    portlet_description = schema.Text(
        title=_("label_subscriber_portlet_description", default="Description"),
        default="",
        required=False,
    )

    newsletter = schema.Choice(
        title=_("label_newsletter_path", default="Path to Newsletter"),
        description=_(
            "help_newsletter_path",
            default="Search for the accordingly newsletter, choose one and apply.",
        ),
        vocabulary="Products.EasyNewsletter.newsletters",
        required=True,
    )

    query_salutation = schema.Bool(
        title=_(
            "label_newletter_show_salutation",
            default="Ask for the salutation of the subscriber",
        ),
        default=True,
        required=False,
    )

    query_name = schema.Bool(
        title=_("label_newletter_show_name", default="Ask for the name of the subscriber"),
        default=True,
        required=False,
    )

    query_firstname = schema.Bool(
        title=_("label_newletter_show_firstname", default="Ask for the firstname as well"),
        default=False,
        required=False,
    )

    query_name_prefix = schema.Bool(
        title=_(
            "label_newletter_show_name_prefix",
            default="Ask for the name prefix (aka title) as well",
        ),
        default=False,
        required=False,
    )

    query_organization = schema.Bool(
        title=_(
            "label_newletter_show_organization",
            default="Display field to enter company/organization of subscriber",
        ),
        default=False,
        required=False,
    )

    show_unsubscribe = schema.Bool(
        title=_(
            "label_newletter_show_unsubscribe_link",
            default="Display an unsubscribe link in portlet footer",
        ),
        default=True,
        required=False,
    )

    additional_info = schema.Text(
        title=_(
            "label_subscriber_portlet_additional_info",
            default=(
                "Additional info, like terms and conditions.\n"
                "It can contain HTML, the following tags are allowed: {0}".format(
                    ", ".join(VALID_TAGS)
                )
            ),
        ),
        default="",
        required=False,
    )


@implementer(INewsletterSubscriberPortlet)
class Assignment(base.Assignment):
    """ """

    portlet_title = "Newsletter"
    portlet_description = ""
    newsletter = ""
    query_name = True
    query_firstname = False
    query_salutation = True
    query_name_prefix = False
    query_organization = False
    show_unsubscribe = True
    additional_info = ""

    def __init__(
        self,
        portlet_title="",
        portlet_description="",
        newsletter="",
        query_name=True,
        query_firstname=False,
        query_salutation=True,
        query_name_prefix=False,
        query_organization=False,
        show_unsubscribe=True,
        additional_info="",
    ):

        self.portlet_title = portlet_title
        self.portlet_description = portlet_description
        self.newsletter = newsletter
        self.query_name = query_name
        self.query_firstname = query_firstname
        self.query_salutation = query_salutation
        self.query_name_prefix = query_name_prefix
        self.query_organization = query_organization
        self.show_unsubscribe = show_unsubscribe
        self.additional_info = additional_info

    @property
    def title(self):
        """ """
        return _("Newsletter subscriber portlet")


class Renderer(base.Renderer):
    """ """

    render = ViewPageTemplateFile("subscriber.pt")

    @property
    def available(self):
        """ """
        return True

    def header(self):
        return self.data.portlet_title

    def description(self):
        return self.data.portlet_description

    def get_newsletter(self):
        """ """
        return self.data.newsletter

    def get_additional_info(self):
        return sanitize_html(self.data.additional_info)


class AddForm(base.AddForm):
    """ """

    schema = INewsletterSubscriberPortlet

    def create(self, data):
        """ """
        return Assignment(
            portlet_title=data.get("portlet_title", ""),
            portlet_description=data.get("portlet_description", ""),
            newsletter=data.get("newsletter", ""),
            query_name=data.get("query_name", ""),
            query_firstname=data.get("query_firstname", ""),
            query_salutation=data.get("query_salutation", ""),
            query_name_prefix=data.get("query_name_prefix", ""),
            query_organization=data.get("query_organization", ""),
            show_unsubscribe=data.get("show_unsubscribe", ""),
            additional_info=data.get("additional_info", ""),
        )


class EditForm(base.EditForm):
    """ """

    schema = INewsletterSubscriberPortlet
    label = _("Edit Newsletter portlet")
    description = _("This portlet displays the subscriber add form of a newsletter.")



"""Invoke tasks for Products.EasyNewsletter."""

from invoke import task


@task
def start(c):
    """Start the Plone instance."""
    c.run("uv run runwsgi instance/etc/zope.ini", env={"LC_ALL": "C.UTF-8"})


@task
def debug(c):
    """Start the Plone instance in debug mode."""
    c.run("uv run runwsgi -d instance/etc/zope.ini")


@task
def shell(c):
    """Open a Zope debug shell."""
    c.run("uv run zconsole debug instance/etc/zope.conf")


@task
def test(c, verbose=False):
    """Run tests."""
    cmd = "uv run pytest"
    if verbose:
        cmd += " -v"
    c.run(cmd)


@task
def create_site(c, site_id="Plone"):
    """Create a new Plone site."""
    import os
    import tempfile

    script = """
from Testing.makerequest import makerequest
import transaction
app = makerequest(app)
from plone.distribution.api import site as site_api
site_api.create(
    app,
    "classic",
    {
        "site_id": "SITE_ID",
        "title": "Products.easynewsletter",
        "description": "A Newsletter and Mailing addon for Plone",
        "default_language": "en",
        "portal_timezone": "UTC",
        "setup_content": False,
    },
)
transaction.commit()
print("Created site: SITE_ID")
""".replace("SITE_ID", site_id)
    with tempfile.NamedTemporaryFile(mode="w", suffix=".py", delete=False) as f:
        f.write(script)
        script_path = f.name
    try:
        c.run(f"uv run zconsole run instance/etc/zope.conf {script_path}")
    finally:
        os.unlink(script_path)


@task
def format(c):
    """Format code with ruff."""
    c.run("uv run ruff format .")
    c.run("uv run ruff check --fix .")


@task
def lint(c):
    """Run linting checks."""
    c.run("uv run ruff check .")



from plone.app.contentmenu.interfaces import IActionsMenu, IActionsSubMenuItem
from plone.app.contentmenu.menu import BrowserMenu, BrowserSubMenuItem
from plone.protect.utils import addTokenToUrl
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.security import checkPermission

from Products.EasyNewsletter.content.newsletter import INewsletter
from Products.EasyNewsletter.content.newsletter_issue import INewsletterIssue


@implementer(IActionsSubMenuItem)
class EasyNewsletterActionsSubMenuItem(BrowserSubMenuItem):
    title = "Newsletter"
    submenuId = "easynewsletter-actions"

    extra = {
        "id": "easynewsletter-actions",
        "li_class": "plonetoolbar-easynewsletter-action",
    }

    order = 40

    @property
    def action(self):
        return self.context.getNewsletter().absolute_url()

    def available(self):
        if (
            checkPermission("cmf.ModifyPortalContent", self.context)
            or checkPermission("cmf.ReviewPortalContent", self.context)
        ) and (INewsletterIssue.providedBy(self.context) or INewsletter.providedBy(self.context)):
            return True
        return False

    def selected(self):
        return False


@implementer(IActionsMenu)
class EasyNewsletterActionsMenu(BrowserMenu):
    def getMenuItems(self, context, request):
        """Return menu item entries in a TAL-friendly form."""
        results = []

        context_state = getMultiAdapter((context, request), name="plone_context_state")
        edit_actions = context_state.actions("folder_easynewsletter")
        if not edit_actions:
            return results

        for action in edit_actions:
            if action["allowed"]:
                aid = action["id"]
                css_class = "actionicon-object_buttons-%s" % aid
                icon = action.get("icon", None)
                modal = action.get("modal", None)
                if modal:
                    css_class += " pat-plone-modal"

                results.append({
                    "title": action["title"],
                    "description": "",
                    "action": addTokenToUrl(action["url"], request),
                    "selected": False,
                    "icon": icon,
                    "extra": {
                        "id": "plone-contentmenu-actions-" + aid,
                        "separator": None,
                        "class": css_class,
                        "modal": modal,
                    },
                    "submenu": None,
                })
        return results



from plone import api

from . import logger


def upgrade(setup_tool=None):
    """ """
    logger.info("Set default aggregation_template on existing Collections")
    brains = api.content.find(portal_type="Collection")
    for brain in brains:
        collection = brain.getObject()
        collection.aggregation_template = "aggregation_generic_listing"



from plone import schema
from plone.dexterity.content import Item
from plone.supermodel import model
from zope.interface import implementer

from Products.EasyNewsletter import _


class INewsletterSubscriber(model.Schema):
    """Marker interface and Dexterity Python Schema for NewsletterSubscriber"""

    salutation = schema.Choice(
        title=_("EasyNewsletter_label_salutation", default="Salutation"),
        description=_("EasyNewsletter_help_salutation", default=""),
        vocabulary="Products.EasyNewsletter.Salutations",
        required=False,
    )

    name_prefix = schema.TextLine(
        title=_("EasyNewsletter_label_name_prefix", default="Name Prefix"),
        description=_("EasyNewsletter_help_name_prefix", default=""),
        default="",
        required=False,
    )

    firstname = schema.TextLine(
        title=_("EasyNewsletter_label_firstname", default="First Name"),
        required=False,
    )

    lastname = schema.TextLine(
        title=_("EasyNewsletter_label_lastname", default="Last Name"), required=False
    )

    organization = schema.TextLine(
        title=_("EasyNewsletter_label_organization", default="Organization"),
        required=False,
    )

    email = schema.TextLine(title=_("EasyNewsletter_label_email", default="Email"), required=True)


@implementer(INewsletterSubscriber)
class NewsletterSubscriber(Item):
    """ """

    @property
    def title(self):
        title = self.email
        prefix = self.name_prefix or ""
        firstname = self.firstname or ""
        lastname = self.lastname or ""
        if self.firstname or self.lastname:
            title += " - " + " ".join([prefix, firstname, lastname])
        return title

    @title.setter
    def title(self, value):
        return



import datetime
import logging

from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.interface import alsoProvides

from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser import BrowserView

log = logging.getLogger("Products.EasyNewsletter: daily-issue")


class DailyIssueView(BrowserView):
    """Creates a new issue of EasyNewsletter and sends it"""

    def has_content(self):
        results = []
        enl = self.context
        sources = enl.content_aggregation_sources
        for source in sources:
            source = source.to_object
            source_results = source.queryCatalog()
            results.extend(source_results)
        return len(results)

    def __generate_id(self):
        return "enlissue_%s" % (datetime.date.today().isoformat())

    def already_sent(self):
        if self.__generate_id() in self.context:
            return True
        else:
            return False

    def create_issue(self):
        id = self.__generate_id()

        try:
            self.issue = api.content.create(type="Newsletter Issue", id=id, container=self.context)
        # If issue already exist, don't create it again
        except BadRequest:
            raise

        self.issue.title = self.context.title
        self.issue.description = self.context.description

        # workaround because of this:
        # https://community.plone.org/t/icontextawaredefaultfactory-has-wrong-context-and-no-acquisition-chain-if-called-in-python/9119
        self.issue.prologue = safe_unicode(self.context.default_prologue)
        self.issue.epilogue = safe_unicode(self.context.default_epilogue)
        self.issue.content_aggregation_sources = self.context.content_aggregation_sources
        self.issue.output_template = self.context.output_template

        # aggregate content for issue:
        # self.context.REQUEST.set("URL", self.issue.absolute_url())
        # self.context.REQUEST.set("ACTUAL_URL", self.issue.absolute_url())
        aggregate_view = getMultiAdapter(
            (self.issue, self.context.REQUEST), name="aggregate-content"
        )
        aggregate_view()

    def send(self):
        if self.issue:
            getMultiAdapter(
                (self.issue, self.context.REQUEST), name="send-issue"
            ).send_issue_immediately()

    def __call__(self):
        if self.request["REQUEST_METHOD"] == "POST":
            alsoProvides(self.request, IDisableCSRFProtection)
            if self.has_content():
                try:
                    self.create_issue()
                    # Accepted. Sending mails
                    self.send()
                    self.request.response.setStatus(200)
                    log.info("Daily issue sended.")
                except BadRequest:
                    # Can't send it twice
                    self.request.response.setStatus(409, "Already Sent Today")
                    log.info("Daily issue already send today, skip!")
            else:
                # Empty issue
                self.request.response.setStatus(204, "Nothing to Send")
                log.info("No data found for daily issue today, skip!")

        elif self.request["REQUEST_METHOD"] == "GET":
            if self.already_sent():
                self.request.response.setStatus(200, "Already Sent Today")
            elif not self.has_content():
                self.request.response.setStatus(204, "Nothing to Send")
            else:
                self.request.response.setStatus(100, "Not sent yet")
        else:
            self.request.response.setStatus(405)
            self.request.response.setHeader("Allow", "GET, POST")


class TriggerDailyIssueView(BrowserView):
    """ """

    def __call__(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        self.request["REQUEST_METHOD"] = "POST"
        view = getMultiAdapter((self.context, self.request), name="daily-issue")
        return view()



import urllib
import uuid

import HTMLParser
import six
import urlparse

from Products.CMFCore.utils import getToolByName


class ENLHTMLParser(HTMLParser.HTMLParser):
    """A simple parser which exchange relative URLs with absolute ones"""

    def __init__(self, context):

        self.context = context
        self.html = ""
        self.image_urls = []
        self.image_number = 0

        HTMLParser.HTMLParser.__init__(self)

    def _encode(self, txt):
        if isinstance(txt, six.text_type):
            plone_utils = getToolByName(self.context, "plone_utils")
            encoding = plone_utils.getSiteEncoding()
            txt = txt.encode(encoding)
        return txt

    def handle_starttag(self, tag, attrs):
        """ """
        self.html += "<%s" % tag

        for attr in attrs:
            if attr[0] == "href":
                try:
                    # split anchor from url
                    baseurl, anchor = urlparse.urldefrag(attr[1])
                    o = self.context.restrictedTraverse(urllib.unquote(baseurl))
                    if getattr(o, "absolute_url", None):
                        url = o.absolute_url()
                    else:
                        # maybe we got a view instead of an traversal object:
                        if getattr(o, "context", None):
                            url = o.context.absolute_url()
                        else:
                            url = attr[1]
                    if anchor:
                        url = "#" + anchor
                except Exception:
                    url = attr[1]
                self.html += ' href="%s"' % self._encode(url)
            else:
                self.html += ' %s="%s"' % (attr)

        self.html += ">"

    def handle_endtag(self, tag):
        """ """
        self.html += "</%s>" % tag

    def handle_data(self, data):
        """ """
        self.html += data

    def handle_charref(self, name):
        self.html += "&#%s;" % name

    def handle_entityref(self, name):
        self.html += "&%s;" % name

    def handle_comment(self, data):
        self.html += "<!--%s-->" % data

    def handle_decl(self, decl):
        self.html += "<!%s>" % decl

    def handle_startendtag(self, tag, attrs):
        """ """
        self.html += "<%s" % tag
        for attr in attrs:
            if attr[0] == "src":
                image_content_id = str(uuid.uuid4())
                if attr[1].startswith(self.context.portal_url()):
                    # content-id must be globaly unique!
                    self.html += ' src="cid:image_%s"' % image_content_id
                    path = attr[1][len(self.context.portal_url()) :]
                    path = "/".join(self.context.getPhysicalPath()) + path
                    self.image_urls.append((path, image_content_id))
                elif "http" in attr[1]:
                    url = attr[1]
                    self.html += ' src="%s"' % self._encode(url)
                else:
                    # content-id must be globaly unique!
                    self.html += ' src="cid:image_%s"' % image_content_id
                    self.image_number += 1
                    self.image_urls.append((attr[1], image_content_id))
            else:
                self.html += ' %s="%s"' % (attr)

        self.html += " />"



from datetime import datetime

from plone import api

from Products.EasyNewsletter import EasyNewsletterMessageFactory as _  # noqa
from Products.EasyNewsletter.content.newsletter_issue import ISendStatus
from Products.Five.browser import BrowserView


class NewsletterIssueStatistics(BrowserView):
    """Show statistics about a newsletter."""

    failed = 0
    successful = 0
    total = 0

    def __call__(self):
        if "reset_statistics" in self.request.form:
            status_adapter = ISendStatus(self.context)
            if status_adapter:
                status_adapter.clear()
                api.portal.show_message(
                    message=_("Newsletter issue statistics have been reset."),
                    request=self.request,
                    type="info",
                )
        return super().__call__()

    def receivers(self):
        status_adapter = ISendStatus(self.context)
        if not status_adapter:
            return
        now = datetime.now()
        records = status_adapter.get_records()
        records = sorted(records, key=lambda x: x.get("status", {}).get("datetime", now))
        self.failed = len(status_adapter.get_keys(successful=False))
        self.successful = len(status_adapter.get_keys(successful=True))
        self.total = len(records)
        return records



# from plone import api


def upgrade(setup_tool=None):
    """ """
    # logger.info("Running upgrade (Python): Update resource registry entries, remove resources, only bundles are needed now.")
    # reload_gs_profile(setup_tool)



from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.interface import Interface

from Products.CMFPlone.interfaces.controlpanel import IMailSchema


class IPortalMailSettings(Interface):
    """PortalMailSettings proxy interface"""


class PortalMailSettings:
    """PortalMailSettings proxy"""

    def __init__(self):
        self.settings = {}

    def __getattr__(self, key):
        self.registry = getUtility(IRegistry)
        reg_mail = self.registry.forInterface(IMailSchema, prefix="plone")
        self.settings["smtp_host"] = reg_mail.smtp_host
        self.settings["smtp_port"] = reg_mail.smtp_port
        self.settings["smtp_userid"] = reg_mail.smtp_userid
        self.settings["smtp_pass"] = reg_mail.smtp_pass
        self.settings["email_from_address"] = reg_mail.email_from_address
        self.settings["email_from_name"] = reg_mail.email_from_name
        self.settings["email_charset"] = reg_mail.email_charset
        return self.settings.get(key)


def get_portal_mail_settings():
    return PortalMailSettings()


def get_email_charset():
    registry = getUtility(IRegistry)
    return registry.get("plone.email_charset", "utf-8")



from plone.protect.interfaces import IDisableCSRFProtection
from zope.interface import alsoProvides

from Products.Five.browser import BrowserView


class ProcessQueue(BrowserView):
    def __call__(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        self.context.send()



import unittest

from App.Common import package_home
from plone import api
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, logout, setRoles
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from zope.component import getMultiAdapter, getSiteManager, getUtility

from Products.CMFPlone.interfaces.controlpanel import IMailSchema
from Products.CMFPlone.tests.utils import MockMailHost
from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter.testing import (
    PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING,
    PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING,
)
from Products.MailHost.interfaces import IMailHost

GLOBALS = globals()
TESTS_HOME = package_home(GLOBALS)


class UnsubscribeFormIntegrationTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.portal_workflow.setDefaultChain("simple_publication_workflow")
        login(self.portal, TEST_USER_NAME)
        registry = getUtility(IRegistry)
        self.mail_settings = registry.forInterface(IMailSchema, prefix="plone")
        self.mail_settings.smtp_host = "localhost"
        self.mail_settings.email_from_address = "portal@plone.test"

        self.newsletter = api.content.create(
            container=self.portal, type="Newsletter", title="ENL 1", id="enl1"
        )
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            id="sub1",
            salutation="mr",
            firstname="Max",
            lastname="Mustermann",
            email="max@example.com",
        )
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            id="sub2",
            salutation="mr",
            firstname="Maxima",
            lastname="Musterfrau",
            email="maxima@example.com",
        )
        self.newsletter.senderEmail = "newsletter@acme.com"
        self.newsletter.senderName = "ACME newsletter"
        self.newsletter.testEmail = "test@acme.com"

        # Set up a mock mailhost
        self.portal._original_MailHost = self.portal.MailHost
        self.portal.MailHost = mailhost = MockMailHost("MailHost")
        self.portal.MailHost.smtp_host = "localhost"
        sm = getSiteManager(context=self.portal)
        sm.unregisterUtility(provided=IMailHost)
        sm.registerUtility(mailhost, provided=IMailHost)
        # We need to fake a valid mail setup
        self.mailhost = self.portal.MailHost
        logout()

    def test_submit_unsubscribe_form(self):
        self.assertSequenceEqual(self.mailhost.messages, [])
        self.portal.REQUEST.form.update({"subscriber": "max@example.com", "test": "submit"})
        view = getMultiAdapter((self.newsletter, self.portal.REQUEST), name="unsubscribe-form")
        view.__call__()
        self.assertEqual(len(self.mailhost.messages), 1)
        self.assertTrue(self.mailhost.messages[0])
        msg = safe_unicode(self.mailhost.messages[0])
        self.assertIn("To: max@example.com", msg)
        self.assertIn("From: portal@plone.test", msg)


class UnsubscribeFormFunctionalTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.unsubscribe_form_url = self.portal_url + "/enl1/unsubscribe-form"
        self.unsubscribe_view_url = self.portal_url + "/enl1/unsubscribe"
        self.browser = Browser(self.portal)
        self.browser.handleErrors = False

        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.portal_workflow.setDefaultChain("simple_publication_workflow")
        self.dummy_page = api.content.create(
            type="Document", id="dummy_page", title="Dummy page", container=self.portal
        )
        self.newsletter = api.content.create(
            container=self.portal, type="Newsletter", title="ENL 1", id="enl1"
        )
        self.newsletter.senderEmail = "newsletter@acme.com"
        self.newsletter.senderName = "ACME newsletter"
        self.newsletter.testEmail = "test@acme.com"
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            id="subscriber1",
            salutation="mr",
            firstname="Max",
            lastname="Mustermann",
            email="max@example.com",
        )
        api.content.create(
            type="Newsletter Subscriber",
            container=self.newsletter,
            id="subscriber2",
            salutation="mr",
            firstname="Maxima",
            lastname="Musterfrau",
            email="maxima@example.com",
        )

        # Commit so that the test browser sees these changes
        import transaction as zt

        zt.commit()
        logout()

    def test_render_unsubscribe_form(self):
        self.browser.open(self.unsubscribe_form_url)
        self.assertTrue("unsubscribe-form" in safe_unicode(self.browser.contents))

    def test_unsubscribe_view(self):
        subscriber1_id = self.newsletter.subscriber1.id
        self.browser.open(
            self.unsubscribe_view_url + "?subscriber=" + self.newsletter.subscriber1.UID()
        )
        self.assertTrue(
            "You have been unsubscribed." in safe_unicode(self.browser.contents),
            "There should be a portal message!",
        )
        self.assertTrue(
            subscriber1_id not in self.newsletter.objectIds(),
            "Subscriber should be delete now!",
        )

        self.browser.open(self.unsubscribe_view_url + "?subscriber=" + self.dummy_page.UID())
        self.assertTrue(
            self.dummy_page.id in self.portal.objectIds(),
            "Dummy page should not be deleted by unsubscribe form!",
        )



# avoid circular import
# from Products.EasyNewsletter import config  # noqa
from zope.i18nmessageid import MessageFactory

EasyNewsletterMessageFactory = MessageFactory("Products.EasyNewsletter")
_ = EasyNewsletterMessageFactory



import base64
import email
import logging
from email.header import decode_header

import cssutils
import six
from plone import api

from Products.CMFPlone.utils import safe_unicode

cssutils.log.setLevel(logging.CRITICAL)


def enable_behavior(content_type=None, behavior=None):
    types_tool = api.portal.get_tool("portal_types")
    fti = types_tool.getTypeInfo(content_type)
    behaviors = fti.getProperty("behaviors")
    behaviors = behaviors + (behavior,)
    return fti.manage_changeProperties(behaviors=behaviors)


def parsed_payloads_from_msg(msg):
    parsed_msg = email.message_from_string(msg)
    parsed_payloads = dict()
    parsed_payloads["to"] = "".join([
        safe_unicode(h[0].strip()) for h in decode_header(parsed_msg.get("To"))
    ])
    parsed_payloads["from"] = "".join([
        safe_unicode(h[0].strip()) for h in decode_header(parsed_msg.get("From"))
    ])
    for part in parsed_msg.walk():
        if part.get_content_type():  # in ["text/plain", "text/html"]:
            payload = part.get_payload()
            if not isinstance(payload, six.string_types):
                continue
            parsed_payloads[part.get_content_type()] = base64.b64decode(part.get_payload())
    return parsed_payloads


def parsed_attachments_from_msg(msg):
    parsed_msg = email.message_from_string(msg)
    parsed_attachments = dict()
    for part in parsed_msg.walk():
        if part.get_content_type():  # in ["text/plain", "text/html"]:
            payload = part.get_payload()
            if not isinstance(payload, six.string_types):
                continue
            parsed_attachments[part.get_filename()] = part["Content-Length"]

    return parsed_attachments



from plone.dexterity.interfaces import IDexterityContent

# from plone import api
from plone.registry.interfaces import IRegistry
from zope.component import getUtility
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from Products.EasyNewsletter import _


class VocabItem:
    def __init__(self, token, value):
        self.token = token
        self.value = value


@implementer(IVocabularyFactory)
class OutputTemplates:
    """ """

    def __call__(self, context):
        items = []

        registry = getUtility(IRegistry)
        output_templates = registry.get("Products.EasyNewsletter.output_templates")
        for key, value in output_templates.items():
            items.append(VocabItem(key, value))
        if not len(items):
            items.append(
                VocabItem(
                    "output_default",
                    _("enl_label_default_output_template", "Default output template"),
                )
            )

        # Fix context if you are using the vocabulary in DataGridField.
        # See https://github.com/collective/collective.z3cform.datagridfield/issues/31:  # NOQA: 501
        if not IDexterityContent.providedBy(context):
            req = getRequest()
            context = req.PARENTS[0]

        # create a list of SimpleTerm items:
        terms = []
        for item in items:
            terms.append(SimpleTerm(value=item.token, token=str(item.token), title=item.value))
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


OutputTemplatesFactory = OutputTemplates()



import unittest

from plone import api
from plone.app.testing import TEST_USER_ID, logout, setRoles
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager

from Products.EasyNewsletter.interfaces import IProductsEasyNewsletterLayer
from Products.EasyNewsletter.testing import (
    PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING,
    PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING,
)
from Products.Five.browser import BrowserView


class ViewletIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.app = self.layer["app"]
        self.request = self.app.REQUEST
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.newsletter = api.content.create(
            container=self.portal, type="Newsletter", id="newsletter"
        )
        self.issue = api.content.create(
            container=self.newsletter, type="Newsletter Issue", id="issue"
        )

    def test_newsletter_actions_is_registered_and_rendered(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        view = BrowserView(self.newsletter, self.request)
        manager_name = "plone.abovecontentbody"
        alsoProvides(self.request, IProductsEasyNewsletterLayer)
        manager = queryMultiAdapter(
            (self.newsletter, self.request, view),
            IViewletManager,
            manager_name,
            default=None,
        )
        self.assertIsNotNone(manager)
        manager.update()
        my_viewlets = [v for v in manager.viewlets if v.__name__ == "newsletter-actions"]  # NOQA: E501
        self.assertEqual(len(my_viewlets), 1)
        self.assertIn("toc-nav newsletter", manager.render())

    def test_newsletter_actions_is_empty_for_non_editors(self):
        logout()
        view = BrowserView(self.newsletter, self.request)
        manager_name = "plone.abovecontentbody"
        alsoProvides(self.request, IProductsEasyNewsletterLayer)
        manager = queryMultiAdapter(
            (self.newsletter, self.request, view),
            IViewletManager,
            manager_name,
            default=None,
        )
        self.assertIsNotNone(manager)
        manager.update()
        my_viewlets = [v for v in manager.viewlets if v.__name__ == "newsletter-actions"]  # NOQA: E501
        self.assertEqual(len(my_viewlets), 1)
        self.assertNotIn("toc-nav newsletter", manager.render())

    def test_newsletter_actions_is_not_available_on_issue(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        view = BrowserView(self.issue, self.request)
        manager_name = "plone.abovecontentbody"
        alsoProvides(self.request, IProductsEasyNewsletterLayer)
        manager = queryMultiAdapter(
            (self.issue, self.request, view),
            IViewletManager,
            manager_name,
            default=None,
        )
        self.assertIsNotNone(manager)
        manager.update()
        my_viewlets = [v for v in manager.viewlets if v.__name__ == "newsletter-actions"]  # NOQA: E501
        self.assertEqual(len(my_viewlets), 0)


class ViewletFunctionalTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])



import csv
import logging

import six
from Acquisition import aq_inner
from plone import api
from plone.i18n.normalizer.interfaces import IIDNormalizer
from zope.component import getUtility

from Products.EasyNewsletter import _
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage

if six.PY2:
    import codecs
else:
    from io import TextIOWrapper

log = logging.getLogger("Products.EasyNewsletter")


def normalize_id(astring):
    return getUtility(IIDNormalizer).normalize(astring)


CSV_HEADER = [
    "salutation",
    "name_prefix",
    "firstname",
    "lastname",
    # u'nl_language',
    "email",
    "organization",
]


class NewsletterSubscribersUpload(BrowserView):
    def __call__(self):
        return self.index()

    def create_subscribers(self):
        """Create newsletter subscribers from uploaded CSV file."""
        # Do nothing if no submit button was hit
        if "form.button.Import" not in self.request.form:
            return

        context = aq_inner(self.context)
        lang = context.Language()
        # plone_utils = getToolByName(self.context, 'plone_utils')
        # encoding = plone_utils.getSiteEncoding()
        existing = []
        subscribers = api.content.find(context=self.context, portal_type="Newsletter Subscriber")
        for subscriber in subscribers:
            existing.append(subscriber.email.lower())

        # messages = IStatusMessage(self.request)
        success = []
        fail = []

        # Show error if no file was specified
        filename = self.request.form.get("csv_upload", None)
        if not filename:
            msg = _("No file specified.")
            IStatusMessage(self.request).addStatusMessage(msg, type="error")
            return self.request.response.redirect(context.absolute_url() + "/subscribers-upload")

        reader = UnicodeReader(filename)
        header = next(reader)
        if header != [x for x in CSV_HEADER]:
            log.info("Got header %s\n Expected:%s" % (header, CSV_HEADER))
            msg = _("Wrong specification of the CSV file. " + "Please correct it and retry.")
            IStatusMessage(self.request).addStatusMessage(msg, type="error")
            return self.request.response.redirect(context.absolute_url() + "/subscribers-upload")

        for subscriber in reader:
            # Check the length of the line
            if len(subscriber) != 6:
                msg = _(
                    "enl_subscriber_upload_wrong_number_of_columns",
                    default="The number of columns in the row is not correct. \
                        It should be 6. Check your CSV file.",
                )
                fail.append({"failure": msg})
            else:
                salutation = subscriber[0]
                name_prefix = subscriber[1]
                firstname = subscriber[2]
                lastname = subscriber[3]
                # nl_language = subscriber[4]
                email = subscriber[4].lower()
                organization = subscriber[5]
                id = normalize_id(email)
                if email in existing:
                    # If subscriber with email exists, update user info
                    sub = api.content.find(context=self.context, email=email)
                    if len(sub) > 1:
                        msg = _(
                            "More than one subscriber with this email "
                            "address existed, subscriber info was NOT "
                            "updated. Check manually!"
                        )
                        fail.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            # 'nl_language': nl_language,
                            "email": email,
                            "organization": organization,
                            "failure": msg,
                        })
                    else:
                        sub = sub[0].getObject()
                        sub.email = email
                        sub.name_prefix = name_prefix
                        sub.firstname = firstname
                        sub.lastname = lastname
                        # sub.nl_language = nl_language
                        sub.organization = organization
                        sub.salutation = salutation
                        sub.title = email + " - " + " ".join([lastname, firstname])
                        sub.reindexObject()
                        msg = _("Email existed, updated subscriber.")
                        success.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            # 'nl_language': nl_language,
                            "email": email,
                            "organization": organization,
                            "success": msg,
                        })
                else:
                    # If it doesn't exist, create subscriber
                    title = email + " - " + " ".join([lastname, firstname])
                    try:
                        api.content.create(
                            type="Newsletter Subscriber",
                            container=self.context,
                            id=id,
                            title=title,
                            description="",
                            language=lang,
                            name_prefix=name_prefix,
                            email=email,
                            firstname=firstname,
                            lastname=lastname,
                            organization=organization,
                            # nl_language=nl_language,
                            salutation=salutation,
                        )
                        # update existing
                        existing.append(email)
                        msg = _("Subscriber created.")
                        success.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            # 'nl_language': nl_language,
                            "email": email,
                            "organization": organization,
                            "success": msg,
                        })

                    except Exception as e:
                        fail.append({
                            "salutation": salutation,
                            "name_prefix": name_prefix,
                            "firstname": firstname,
                            "lastname": lastname,
                            # 'nl_language': nl_language,
                            "email": email,
                            "organization": organization,
                            "failure": "An error occured while creating this subscriber: %s"
                            % str(e),
                        })

        return {"success": success, "fail": fail}


class UTF8Recoder:
    """
    Iterator that reads an encoded stream and reencodes the input to UTF-8
    """

    def __init__(self, f, encoding):
        if six.PY2:
            self.reader = codecs.getreader(encoding)(f)
        else:
            self.reader = TextIOWrapper(f, encoding=encoding)

    def __iter__(self):
        return self

    def __next__(self):
        data = next(self.reader)
        if six.PY2:
            data = data.encode("utf-8")
        return data

    next = __next__  # BBB for Python2


class UnicodeReader:
    """
    A CSV reader which will iterate over lines in the CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        f = UTF8Recoder(f, encoding)
        self.reader = csv.reader(f, dialect=dialect, **kwds)

    def __next__(self):
        row = next(self.reader)
        return row

    next = __next__  # BBB for Python2

    def __iter__(self):
        return self



import codecs
import csv
import tempfile

import six
from Acquisition import aq_inner
from plone import api

from Products.CMFPlone.utils import safe_encode, safe_unicode
from Products.Five.browser import BrowserView

CSV_HEADER = [
    "salutation",
    "name_prefix",
    "firstname",
    "lastname",
    # u"nl_language",
    "email",
    "organization",
]


class NewsletterSubscribersDownload(BrowserView):
    def __call__(self):
        context = aq_inner(self.context)
        # Create CSV file
        filename = tempfile.mktemp()
        if six.PY2:
            file = open(filename, "wb")
            csvWriter = UnicodeWriter(
                file, {"delimiter": ",", "quotechar": '"', "quoting": csv.QUOTE_MINIMAL}
            )
        else:
            file = open(filename, "w", newline="")
            csvWriter = csv.writer(
                file,
                dialect="excel",
                delimiter=",",
                quotechar='"',
                quoting=csv.QUOTE_MINIMAL,
            )

        csvWriter.writerow([x for x in CSV_HEADER])
        for subscriber in api.content.find(
            portal_type="Newsletter Subscriber", context=self.context, sort_on="email"
        ):
            obj = subscriber.getObject()
            csvWriter.writerow([
                obj.salutation,
                obj.name_prefix,
                obj.firstname,
                obj.lastname,
                # obj.nl_language,
                obj.email,
                obj.organization,
            ])
        file.close()

        with open(filename) as f:
            data = f.read()
        data = safe_encode(data)
        # Create response
        response = context.REQUEST.response
        response.addHeader(
            "Content-Disposition", "attachment; filename=easynewsletter-subscribers.csv"
        )
        response.addHeader("Content-Type", "text/csv")
        response.addHeader("Content-Length", "%d" % len(data))
        response.addHeader("Pragma", "no-cache")
        response.addHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0, public")
        response.addHeader("Expires", "0")

        # Return CSV data
        return data


class UnicodeWriter:
    """
    A CSV writer which will write rows to CSV file "f",
    which is encoded in the given encoding.
    """

    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
        # Redirect output to a queue
        self.queue = six.StringIO()
        self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
        self.stream = f
        self.encoder = codecs.getincrementalencoder(encoding)()

    def writerow(self, row):
        # row = [s or '' for s in row]
        row = [safe_encode(s) or b"" for s in row]
        self.writer.writerow(row)
        data = self.queue.getvalue()
        data = safe_unicode(data)
        data = self.encoder.encode(safe_unicode(data))
        # write to the target stream
        self.stream.write(data)
        # empty queue
        self.queue.truncate(0)

    def writerows(self, rows):
        for row in rows:
            self.writerow(row)



from plone.protect.interfaces import IDisableCSRFProtection
from zope.interface import alsoProvides

from Products.CMFPlone.resources import add_resource_on_request
from Products.EasyNewsletter import _  # noqa
from Products.Five.browser import BrowserView


class NewsletterIssue(BrowserView):
    def __call__(self):
        add_resource_on_request(self.request, "iframeResizer")
        return self.index()

    @property
    def here_url(self):
        return self.context.absolute_url()

    def refresh_issue(self, REQUEST=None):  # noqa
        """Refresh the aggregate body when using collections."""
        alsoProvides(self.request, IDisableCSRFProtection)
        self.context.loadContent()
        self.request.response.redirect(self.context.absolute_url())



import logging

from zope.interface import implementer

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import INonInstallable


@implementer(INonInstallable)
class HiddenProfiles:
    def getNonInstallableProfiles(self):  # noqa
        """Hide uninstall profile from site-creation and quickinstaller"""
        return [
            # 'Products.EasyNewsletter:install-base',
            "Products.EasyNewsletter:uninstall",
        ]

    def getNonInstallableProducts(self):
        return [
            "Products.EasyNewsletter.upgrades",
        ]


# The profile id of your package:
# PROFILE_ID = 'profile-Products.EasyNewsletter:install-base'
PROFILE_ID = "profile-Products.EasyNewsletter:default"


def add_catalog_indexes(context, logger=None):
    """Method to add our wanted indexes to the portal_catalog.

    @parameters:

    When called from the import_various method below, 'context' is
    the plone site and 'logger' is the portal_setup logger.  But
    this method can also be used as upgrade step, in which case
    'context' will be portal_setup and 'logger' will be None.
    """
    if logger is None:
        # Called as upgrade step: define our own logger.
        logger = logging.getLogger("Products.EasyNewsletter")

    # Run the catalog.xml step as that may have defined new metadata
    # columns.  We could instead add <depends name="catalog"/> to
    # the registration of our import step in zcml, but doing it in
    # code makes this method usable as upgrade step as well.  Note that
    # this silently does nothing when there is no catalog.xml, so it
    # is quite safe.
    setup = getToolByName(context, "portal_setup")
    setup.runImportStepFromProfile(PROFILE_ID, "catalog")

    catalog = getToolByName(context, "portal_catalog")
    indexes = catalog.indexes()
    # Specify the indexes you want, with ('index_name', 'index_type')

    wanted = (
        ("fullname", "FieldIndex"),
        ("firstname", "FieldIndex"),
        ("lastname", "FieldIndex"),
        ("nl_language", "FieldIndex"),
        ("email", "FieldIndex"),
        ("organization", "FieldIndex"),
    )

    indexables = []
    for name, meta_type in wanted:
        if name not in indexes:
            catalog.addIndex(name, meta_type)
            indexables.append(name)
            logger.info("Added %s for field %s.", meta_type, name)
    if len(indexables) > 0:
        logger.info("Indexing new indexes %s.", ", ".join(indexables))
        catalog.manage_reindexIndex(ids=indexables)


def import_various(context):
    """Import step for configuration that is not handled in xml files."""
    # Only run step if a flag file is present
    if context.readDataFile("Products.EasyNewsletter-default.txt") is None:
        return
    logger = context.getLogger("Products.EasyNewsletter")
    site = context.getSite()
    add_catalog_indexes(site, logger)



from plone import api
from plone.dexterity.interfaces import IDexterityContent
from zope.component import subscribers
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from Products.EasyNewsletter.interfaces import IReceiversGroupFilter


@implementer(IVocabularyFactory)
class PloneGroups:
    """ """

    def __call__(self, context):

        results = []
        group_properties = dict()
        groups = api.group.get_groups()

        # Fix context if you are using the vocabulary in DataGridField.
        # See https://github.com/collective/collective.z3cform.datagridfield/issues/31:  # NOQA: 501
        if not IDexterityContent.providedBy(context):
            req = getRequest()
            context = req.PARENTS[0]

        for group in groups:
            group_id = group.getId()
            group_properties[group_id] = {
                "title": group.getGroupTitleOrName(),
                "email": group.getProperty("email"),
            }
        results = [(id, property["title"]) for id, property in group_properties.items()]

        # run registered group filter:
        for subscriber in subscribers([self], IReceiversGroupFilter):
            results = subscriber.filter(results)

        # create a list of SimpleTerm items:
        terms = []
        for item in results:
            terms.append(SimpleTerm(value=item[0], token=item[0], title=item[1]))
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


PloneGroupsFactory = PloneGroups()



from collective.zamqp.consumer import Consumer
from collective.zamqp.interfaces import IProducer
from collective.zamqp.producer import Producer
from plone import api
from zope.component import getUtility
from zope.interface import Interface, implementer

from Products.EasyNewsletter.queue.interfaces import IIssueQueue

QUEUE_NAME = "Products.EasyNewsletter.queue"


class IMailProcessingMessage(Interface):
    """Marker interface for Mail processing  message"""


class MailProcessingProducer(Producer):
    """Produces Mail processing tasks"""

    connection_id = "superuser"
    serializer = "msgpack"
    queue = QUEUE_NAME
    routing_key = QUEUE_NAME

    durable = True


class MailProcessingConsumer(Consumer):
    """Consumes Mail processing tasks"""

    connection_id = "superuser"
    marker = IMailProcessingMessage
    queue = QUEUE_NAME
    routing_key = QUEUE_NAME

    durable = True


def process_message(message, event):
    """Handle messages received through consumer."""
    uuid = message.header_frame.correlation_id
    context = api.content.get(UID=uuid)

    # send out emails
    context.send()

    # Send ACK
    message.ack()


@implementer(IIssueQueue)
class ZAMQPIssueQueue:
    def start(self, context):
        """Queues issue for sendout through collective.zamqp."""
        kwargs = {}
        producer = getUtility(IProducer, name=QUEUE_NAME)
        producer.register()
        producer.publish(kwargs, correlation_id=api.content.get_uuid(context))



from Products.Five.browser import BrowserView


class NewsletterIssueSendForm(BrowserView):
    def __call__(self):
        return self.index()

    @property
    def sender_name(self):
        newsletter = self.context.get_newsletter()
        return self.request.get("sender_name") or newsletter.sender_name

    @property
    def sender_email(self):
        newsletter = self.context.get_newsletter()
        return self.request.get("sender_email") or newsletter.sender_email

    @property
    def subject(self):
        return self.request.get("subject") or self.context.title

    @property
    def test_receiver(self):
        newsletter = self.context.get_newsletter()
        return self.request.get("test_receiver") or newsletter.test_email



import unittest
from datetime import datetime

from plone import api
from plone.app.testing import TEST_USER_ID, setRoles

from Products.EasyNewsletter.content.newsletter_issue import ISendStatus
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class SendStatusTests(unittest.TestCase):
    """"""

    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.newsletter = api.content.create(
            container=self.portal,
            type="Newsletter",
            id="newsletter",
        )
        self.issue = api.content.create(
            container=self.newsletter,
            type="Newsletter Issue",
            id="issue",
        )

    @property
    def receivers(self):
        return [
            {
                "email": "john@example.com",
                "fullname": "John Doe",
                "firstname": "John",
                "lastname": "Doe",
                "salutation": "Dear Mr.",
                "status": {
                    "successful": True,
                    "error": None,
                    "datetime": datetime.now(),
                },
            },
            {
                "email": "Joe@example.com",
                "fullname": "Joe Doe",
                "firstname": "Joe",
                "lastname": "Doe",
                "salutation": "Dear Mr.",
                "status": {
                    "successful": False,
                    "error": "Some error message",
                    "datetime": datetime.now(),
                },
            },
            {
                "email": "mary@example.com",
                "fullname": "Mary Doe",
                "firstname": "Mary",
                "lastname": "Doe",
                "salutation": "Dear Mrs.",
            },
        ]

    def test_add_records(self):
        """Validate the 'add_records' method."""
        status_adapter = ISendStatus(self.issue)
        status_adapter.add_records(self.receivers)
        self.assertEqual(len(status_adapter.get_keys()), 3)

        # Adding the same records will not give more items.
        status_adapter.add_records(self.receivers)
        self.assertEqual(len(status_adapter.get_keys()), 3)

        # Adding a new record will reurn more items.
        status_adapter.add_records([{"email": "info@example.com"}])
        self.assertEqual(len(status_adapter.get_keys()), 4)

    def test_get_records(self):
        """Validate the 'get_records' method."""
        status_adapter = ISendStatus(self.issue)
        status_adapter.add_records(self.receivers)
        self.assertEqual(len(status_adapter.get_records()), 3)
        self.assertEqual(len(status_adapter.get_records(successful=True)), 1)
        self.assertEqual(len(status_adapter.get_records(successful=False)), 1)

    def test_get_keys(self):
        """Validate the 'get_keys' method."""
        status_adapter = ISendStatus(self.issue)
        status_adapter.add_records(self.receivers)
        self.assertEqual(len(status_adapter.get_keys()), 3)
        self.assertEqual(len(status_adapter.get_keys(successful=True)), 1)
        self.assertEqual(len(status_adapter.get_keys(successful=False)), 1)

    def test_reset_statistics_making_master(self):
        """Validate statistics are reset when issue is made as master."""
        status_adapter = ISendStatus(self.issue)
        status_adapter.add_records(self.receivers)
        self.assertEqual(len(status_adapter.get_keys()), 3)
        request = self.issue.REQUEST
        request["enlwf_guard"] = True
        api.content.transition(obj=self.issue, transition="make_master")
        self.assertEqual(len(status_adapter.get_keys()), 0)
        request["enlwf_guard"] = False



# -*- coding: utf-8 -*-
# flake8: noqa

from plone import api
from Products.EasyNewsletter import _
from zope.i18nmessageid import MessageFactory

import re


# see __init__.py
# _ = MessageFactory('Products.EasyNewsletter')

PROJECTNAME = "EasyNewsletter"

ENL_ISSUE_TYPES = ["ENLIssue", "Newsletter Issue"]
ENL_EDITHELPER_TYPES = ["EasyNewsletter", "ENLIssue", "Newsletter", "Newsletter Issue"]

PLACEHOLDERS = ["UNSUBSCRIBE", "SUBSCRIBER_SALUTATION"]

AGG_SOURCES_INFOS = "agg_sources_infos"

SALUTATION = {
    "ms": _("label_salutation_ms", "Ms."),
    "mr": _("label_salutation_mr", "Mr."),
}

# NL_LANGUAGE = DisplayList((
#     ('', _(u'label_choose_nl_language', 'Choose language...')),
#     ('de', _(u'label_salutation_de', 'DE')),
#     ('en', _(u'label_salutation_en', 'EN')),
#     ('fr', _(u'label_salutation_fr', 'FR')),
# ))

MESSAGE_CODE = {
    "email_added": _(
        "email_added",
        default="Your email has been registered. A confirmation email was"
        " sent to your address. Please check your inbox and click "
        " on the link in the email in order to confirm your"
        " subscription.",
    ),
    "invalid_email": _("invalid_email", default="Please enter a valid email address."),
    "email_exists": _("email_exists", default="Your email address is already registered."),
    "invalid_hashkey": _("invalid_hashkey", default="Please enter a valid email address."),
    "subscription_confirmed": _(
        "subscription_confirmed",
        default="Your subscription was successfully confirmed.",
    ),
}


EMAIL_RE = re.compile(r"(?:^|\s)[-a-z0-9_.]+@(?:[-a-z0-9]+\.)+[a-z]{2,63}(?:\s|$)", re.IGNORECASE)


DEFAULT_SUBSCRIBER_CONFIRMATION_MAIL_SUBJECT = _("Confirm your subscription on ${portal_url}")

DEFAULT_SUBSCRIBER_CONFIRMATION_MAIL_TEXT = """\
You subscribe to the ${newsletter_title}.\n\n
Your registered email is: ${subscriber_email}\n
Please click on the link to confirm your subscription: \n
${confirmation_url}"""



import unittest
from datetime import datetime, timedelta

from App.Common import package_home
from plone import api
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, setRoles

from Products.EasyNewsletter.interfaces import IIssueDataFetcher
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

GLOBALS = globals()
TESTS_HOME = package_home(GLOBALS)


class PlaceholderIntegrationTests(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.portal_url()
        # create EasyNewsletter instance and add some subscribers
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.newsletter = api.content.create(
            container=self.portal,
            type="Newsletter",
            id="enl1",
            title="ENL 1",
            sneder_email="newsletter@acme.com",
            sender_name="ACME newsletter",
            test_email="test@acme.com",
        )
        api.content.create(
            container=self.newsletter,
            type="Newsletter Issue",
            id="issue",
            title="This is a very long newsletter issue title with special "
            "characters such as äüö. Will this really work?",
        )

    def test_placeholder_month_year(self):
        receiver = {
            "email": "john@example.com",
            "fullname": "John Doe",
            "firstname": "John",
            "lastname": "Doe",
            "salutation": "Dear Mr.",
        }
        html = """
        <h1>Plone ist toll!</h1>
        {{ SUBSCRIBER_SALUTATION }}
        {{ month }}/{{ year }}
        """
        now = datetime.now()
        effective_date = now + timedelta(2)
        self.newsletter.issue.effective_date = effective_date
        self.assertNotEqual(self.newsletter.issue.modified(), self.newsletter.issue.effective())
        issue_data_fetcher = IIssueDataFetcher(self.newsletter.issue)
        issue_data = issue_data_fetcher.personalize(receiver, html)
        self.assertIn(
            f"{self.newsletter.issue.effective().month()}/{self.newsletter.issue.effective().year()}",
            issue_data,
        )

    def test_placeholder_calendar_week(self):
        receiver = {
            "email": "john@example.com",
            "fullname": "John Doe",
            "firstname": "John",
            "lastname": "Doe",
            "salutation": "Dear Mr.",
        }
        html = """
        <h1>Plone ist toll!</h1>
        {{ SUBSCRIBER_SALUTATION }}
        {{ calendar_week }}
        """
        now = datetime.now()
        effective_date = now + timedelta(2)
        self.newsletter.issue.effective_date = effective_date
        self.assertNotEqual(self.newsletter.issue.modified(), self.newsletter.issue.effective())
        issue_data_fetcher = IIssueDataFetcher(self.newsletter.issue)
        issue_data = issue_data_fetcher.personalize(receiver, html)
        self.assertIn(self.newsletter.issue.effective().strftime("%V"), issue_data)



from zope.interface import Attribute, Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer


class IProductsEasyNewsletterLayer(IDefaultBrowserLayer):
    """ """


class IIssueView(Interface):
    """ """

    def get_unpersonalized_body(self):
        """ """


class ISubscriberSource(Interface):
    """Interface for utilities providing a list of subscribers.
    Such an utility should be registered as named utility since
    the name of the utility is referenced inside the Newsletter
    implementation as a reference to an external subscriber source.
    """

    def getSubscribers(newsletter):
        """Return a list of subscribers for a given newsletter.
        Each subscriber should be represented as dictionary
        with keys 'email' and 'fullname'.
        """


class IReceiversMemberFilter(Interface):
    """Marker interface for ReceiverMemberFilters subscribers.
    This filter runs befor the available member selections list is
    displayed, so you can use it to add or remove member entries
    from the selection list.
    """


class IReceiversGroupFilter(Interface):
    """Marker interface for ReceiverGroupFilters subscribers.
    This filter runs befor the available group selections list is
    displayed, so you can use it to add or remove group entries for
    the selection list.
    """


class IReceiversPostSendingFilter(Interface):
    """Marker interface for ReceiversPostSendingFilter subscribers.
    This filter runs after selection of members and groups, before
    sending. So this is the final line, to add or filter out
    recievers.
    """


class IENLRegistrationTool(Interface):
    """Marker interface for ENL registration utility"""


class IIssueDataFetcher(Interface):
    """Provides personalized issue data"""

    def __call__(receiver):
        """return issue data dict-like object with keys:

        ``subject_header``
            the subject header line

        - ``body_plain``
            the plain text body of the email

        - ``body_html``
            the html body of the email

        - ``images_to_attach``
            list of Image Plone content objects
        """

    def preview_html():
        """html meant for preview of the newsletter in browser."""


class IBeforePersonalizationEvent(Interface):
    data = Attribute("issue specific data")



import unittest

from plone.app.testing import TEST_USER_ID, setRoles
from plone.behavior.interfaces import IBehavior
from zope.component import getUtility

from Products.EasyNewsletter.behaviors.external_subscriber_sources import (
    IExternalSubscriberSourcesMarker,
)
from Products.EasyNewsletter.testing import PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING


class ExternalSubscriberSourcesIntegrationTest(unittest.TestCase):
    layer = PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_behavior_external_subscriber_sources(self):
        behavior = getUtility(IBehavior, "Products.EasyNewsletter.external_subscriber_sources")
        self.assertEqual(
            behavior.marker,
            IExternalSubscriberSourcesMarker,
        )



# example usage: ./bin/instance run import_enl_stats.py --errors-filenam=missing.csv --issue-path=/enl-dx-clean/test-newsletter-issue /Plone

import argparse
import logging
import sys
from datetime import datetime

from plone import api
from transaction import commit
from zope.site.hooks import setSite

from Products.EasyNewsletter.content.newsletter_issue import ISendStatus

# from Acquisition import aq_inner
# from Products.CMFPlone.utils import safe_unicode


def get_base_parser():
    parser = argparse.ArgumentParser(description=SCRIPTNAME)
    parser.add_argument(
        "plonesite_path",
        default="/Plone",
        # metavar='"/Plone"',
        action="store",
        # dest="plonesite_path",
        type=str,
        nargs="?",
        help="Path to the Plone site",
    )
    parser.add_argument(
        "-b",
        "--commit_batch_size",
        type=int,
        action="store",
        dest="commit_batch_size",
        default=100,
        metavar="N",
        nargs="?",
        help="Do a transaction commit every N items. Default: 100",
    )
    parser.add_argument(
        "--hostname",
        action="store",
        dest="hostname",
        default="nohost",
        nargs="?",
        help="Define the hostname, Plone should use for creating urls. Default: nohost",  # NOQA: E501
    )
    parser.add_argument(
        "--protocol",
        action="store",
        dest="protocol",
        default="http",
        nargs="?",
        help="Define the protocol, Plone should use for creating urls. Default: http",  # NOQA: E501
    )
    parser.add_argument(
        "--quiet", action="store_true", help="Only show errors. Useful for cronjobs.",
    )
    parser.add_argument(
        "--verbose", action="store_true", help="Show debug infos.",
    )
    return parser


def get_logger(name, args):
    log = logging.getLogger(name)
    if args.quiet:
        log.setLevel(logging.ERROR)
    elif args.verbose:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.INFO)

    handler = logging.StreamHandler(sys.stdout)
    formatter = logging.Formatter(
        "%(asctime)s %(levelname)s %(name)s: %(message)s", datefmt="%m/%d/%Y %H:%M:%S"
    )
    handler.setFormatter(formatter)
    log.addHandler(handler)
    return log


class BaseScriptWrapper:
    """
    """

    def __init__(self, app, args):
        """
        """
        plonesite_path = args.plonesite_path
        portal = app.unrestrictedTraverse(plonesite_path)  # noqa: F821
        setSite(portal)
        self.portal = portal
        self.request = self.portal.REQUEST
        self.request["PARENTS"] = [self.portal]
        self.commit_batch_size = args.commit_batch_size
        protocol = args.protocol
        hostname = args.hostname
        port = protocol == "http" and "80" or "443"

        self.request.setServerURL(
            protocol=protocol, hostname=hostname, port=port,
        )
        self.request.setVirtualRoot("")
        self.args = args



#################
## customization:
#################


SCRIPTNAME = "import_enl_stats: "


parser = get_base_parser()

parser.add_argument(
    '--errors-filename',
    action='store',
    dest="errors_filename",
    default="missing.csv",
    type=str,
    help='CSV filename to import sendiong error addresses from.',
)

parser.add_argument(
    '--issue-path',
    action='store',
    dest="issue_path",
    type=str,
    help='ENL issue where we import the stats to.',
)

# remove -c script_name from args before argparse runs:
if "-c" in sys.argv:
    index = sys.argv.index("-c")
    del sys.argv[index]
    del sys.argv[index]

args = parser.parse_args()
log = get_logger(SCRIPTNAME, args)


class ScriptWrapper(BaseScriptWrapper):
    """
    """

    def run(self):
        enl_issue = api.content.get(args.issue_path)
        enl = enl_issue.get_newsletter()
        subscribers = [obj.email for obj in enl.objectValues() if obj.portal_type == "Newsletter Subscriber"]
        status_adapter = ISendStatus(enl_issue)

        with open(args.errors_filename) as f:
            error_lines = f.readlines()
        error_emails = []
        for line in error_lines:
            error_emails.append(line.strip('\n'))

        successful_subscribers = set(subscribers).difference(set(error_emails))
        records = []
        record_status = {'successful': True, 'error': None, 'datetime': datetime.now()}
        for sub in successful_subscribers:
            records.append({"email": sub, "status": record_status})
            print(f"Add subscriber '{sub}' as successful to stattistics.")
        status_adapter.add_records(records)
        commit()


if __name__ == "__main__":
    script_wrapper = ScriptWrapper(app, args,)
    script_wrapper.run()



from copy import copy

from Acquisition import aq_inner
from plone import api
from plone.app.textfield import RichTextValue

# from transaction import commit
from zope.annotation import IAnnotations

from Products.CMFPlone.utils import safe_unicode
from Products.EasyNewsletter import _
from Products.EasyNewsletter.config import AGG_SOURCES_INFOS
from Products.Five.browser import BrowserView


class NewsletterIssueAggregateContent(BrowserView):
    def __call__(self):
        text = self.render_aggregation_sources()
        self.context.text = RichTextValue(
            raw=text,
            mimeType="text/html",
            outputMimeType="text/html",
        )
        api.portal.show_message(
            message=_("Newsletter content successfully aggregated."),
            request=self.request,
            type="info",
        )
        self.request.response.setHeader("Pragma", "no-cache")
        self.request.response.setHeader("Cache-Control", "no-cache")
        return self.request.response.redirect(self.context.absolute_url(), status=301)

    def render_aggregation_sources(self):
        """ """
        results_text = ""
        portal = api.portal.get()
        sources = self.context.content_aggregation_sources
        if not sources:
            return results_text

        # clear AGG_SOURCES_INFOS in annotations:
        annotations = IAnnotations(aq_inner(self.context))
        if AGG_SOURCES_INFOS in annotations:
            del annotations[AGG_SOURCES_INFOS]

        for source in sources:
            source_obj = source.to_object
            sresults = source_obj.queryCatalog(b_size=30000)
            if not sresults:
                continue
            # Convert CatalogContentListingObject instances to raw catalog
            # brains, which have __allow_access_to_unprotected_subobjects__
            # and work correctly in restricted Python (skin templates).
            sresults = [getattr(r, "_brain", r) for r in sresults]
            result_info = {
                "id": source_obj.id,
                "title": source_obj.Title(),
                "description": source_obj.Description(),
                "text": source_obj.text and source_obj.text.output,
                "uid": source_obj.UID(),
                "portal_type": sresults[0].portal_type,
                "brains": sresults,
                "brains_count": len(sresults),
                "lang": "de",
                "LANGUAGE": "de",
            }

            template_id = source_obj.aggregation_template
            if not template_id:
                template_id = "aggregation_generic_listing"
            template_obj = portal.restrictedTraverse(str(template_id))
            results_text += template_obj.render(result_info=result_info)
            self.store_source_info_in_annotation(result_info)
        return safe_unicode(results_text)

    def store_source_info_in_annotation(self, source_info):
        """ """
        annotations = IAnnotations(aq_inner(self.context))
        if AGG_SOURCES_INFOS not in annotations:
            annotations[AGG_SOURCES_INFOS] = []
        sinfo = copy(source_info)
        # remove brains, we can't pickle them
        del sinfo["brains"]
        annotations[AGG_SOURCES_INFOS].append(sinfo)



from persistent.dict import PersistentDict
from plone import schema
from plone.app import textfield
from plone.app.z3cform.widget import SingleCheckBoxBoolFieldWidget
from plone.autoform import directives
from plone.dexterity.content import Container
from plone.namedfile import field as namedfile
from plone.supermodel import model
from z3c import relationfield
from zope.annotation.interfaces import IAnnotations
from zope.interface import Interface, implementer, provider
from zope.schema.interfaces import IContextAwareDefaultFactory

from Products.EasyNewsletter import _

from .newsletter import INewsletter, get_content_aggregation_sources_base_path

SEND_STATUS_KEY = "PRODUCTS_EASYNEWSLETTER_SEND_STATUS"


@provider(IContextAwareDefaultFactory)
def get_default_output_template(parent):
    """get ouput template from parent Newsletter"""
    if INewsletter.providedBy(parent) and parent.__parent__:
        return parent.output_template


@provider(IContextAwareDefaultFactory)
def get_default_prologue(parent):
    """get prologue from parent Newsletter"""
    prologue_output = ""
    if INewsletter.providedBy(parent) and parent.__parent__ and parent.default_prologue:
        prologue_output = parent.default_prologue.raw
    default_prologue = textfield.RichTextValue(
        raw=prologue_output,
        mimeType="text/html",
        outputMimeType="text/x-plone-outputfilters-html",
    )
    return default_prologue


@provider(IContextAwareDefaultFactory)
def get_default_epilogue(parent):
    """get epilogue from parent Newsletter"""
    epilogue_output = ""
    if INewsletter.providedBy(parent) and parent.__parent__ and parent.default_epilogue:
        epilogue_output = parent.default_epilogue.raw
    default_epilogue = textfield.RichTextValue(
        raw=epilogue_output,
        mimeType="text/html",
        outputMimeType="text/x-plone-outputfilters-html",
    )
    return default_epilogue


@provider(IContextAwareDefaultFactory)
def get_default_content_aggregation_sources(parent):
    """get content_aggregation_sources from parent Newsletter"""
    if INewsletter.providedBy(parent) and parent.__parent__:
        return parent.content_aggregation_sources


class INewsletterIssue(model.Schema):
    """Marker interface and Dexterity Python Schema for NewsletterIssue"""

    model.fieldset(
        "customizations",
        label=_("Customizations"),
        fields=[
            "prologue",
            "epilogue",
            "content_aggregation_sources",
            "exclude_all_subscribers",
            "banner",
            "hide_image",
            "output_template",
        ],
    )

    directives.widget(
        "content_aggregation_sources",
        pattern_options={
            "basePath": get_content_aggregation_sources_base_path,
            "selectableTypes": ["Collection"],
        },
    )
    content_aggregation_sources = relationfield.schema.RelationList(
        title=_(
            "ENL_content_aggregation_sources_label",
            default="Content aggregation sources",
        ),
        description=_(
            "ENL_content_aggregation_sources_desc",
            default="Choose sources to aggregate newsletter content from.",
        ),
        value_type=relationfield.schema.RelationChoice(
            title="content_aggretation_source",
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        defaultFactory=get_default_content_aggregation_sources,
        required=False,
    )

    # Make sure you import:
    # plone.app.z3cform.widget.SingleCheckBoxBoolFieldWidget
    directives.widget(exclude_all_subscribers=SingleCheckBoxBoolFieldWidget)
    exclude_all_subscribers = schema.Bool(
        title=_("ENL_label_excludeAllSubscribers", default="Exclude all subscribers"),
        description=_(
            "ENL_help_excludeAllSubscribers",
            default="If checked, the newsletter/mailing will not be send  \
                to all subscribers inside the newsletter. Changing this \
                setting does not affect already existing issues.",
        ),
        required=False,
        default=False,
    )

    output_template = schema.Choice(
        title=_("enl_label_output_template", default="Output template"),
        description=_(
            "enl_help_output_template",
            default="Choose the template to render the email. ",
        ),
        vocabulary="Products.EasyNewsletter.OutputTemplates",
        defaultFactory=get_default_output_template,
        required=True,
    )

    # Make sure to import: plone.app.textfield
    prologue = textfield.RichText(
        title=_("ENL_label_default_header", default="Prologue"),
        description=_(
            "ENL_description_text_header",
            default="The default prologue text. This is used as a default \
                for new issues. You can use placeholders like\
                {{subscriber_salutation}} and {{unsubscribe}} here.",
        ),
        defaultFactory=get_default_prologue,
        required=False,
    )

    # Make sure to import: plone.app.textfield
    epilogue = textfield.RichText(
        title=_("ENL_label_default_footer", default="Epilogue"),
        description=_(
            "ENL_description_text_footer",
            default="The default epilogue text. This is used as a default \
                for new issues. You can use placeholders like\
                {{subscriber_salutation}} and {{unsubscribe}} here.",
        ),
        defaultFactory=get_default_epilogue,
        required=False,
    )

    # Make sure you import:
    # plone.app.z3cform.widget.SingleCheckBoxBoolFieldWidget
    directives.widget(hide_image=SingleCheckBoxBoolFieldWidget)
    hide_image = schema.Bool(
        title=_("label_issueHideImage", default="Hide banner image."),
        description=_(
            "enl_issue_help_hide_image",
            default="If checked, the banner image defined on newsletter \
                    or on this issue will not be used.",
        ),
        required=False,
        default=False,
        readonly=False,
    )

    banner = namedfile.NamedBlobImage(
        title=_("ENL_image_label", default="Banner image"),
        description=_(
            "ENL_image_desc",
            default="Banner image, you can include in the templates by"
            + "\n adding the {{banner}} placeholder into it."
            + " By default it should be 600x200 pixel.",
        ),
        required=False,
    )

    # directives.order_after(content_aggregation_source="IBasic.title")
    # directives.order_after(output_template="IRichText.text")


def context_property(name):
    def getter(self):
        return getattr(self.context, name)

    def setter(self, value):
        setattr(self.context, name, value)

    def deleter(self):
        delattr(self.context, name)

    return property(getter, setter, deleter)


@implementer(INewsletterIssue)
class NewsletterIssue(Container):
    """ """

    context_property("content_aggregation_sources")

    def get_newsletter(self):
        return self.__parent__

    # bbb to support ATCT way, needs to be removed in v5.x:
    getNewsletter = get_newsletter

    def has_image(self):
        has_image = bool(self.get_image_src())
        return has_image

    def has_logo(self):
        enl = self.get_newsletter()
        has_logo = getattr(enl.aq_explicit, "logo", None)
        return has_logo

    # XXX we should cache this call, it's called twice
    def get_image_src(self):
        """find banner image, if not set on Issue we use the one from the Newsletter"""
        if self.hide_image:
            return ""
        scales = self.restrictedTraverse("@@images")
        scale = scales.scale("banner")
        if scale:
            return scale.url
        enl = self.get_newsletter()
        scales = enl.restrictedTraverse("@@images")
        scale = scales.scale("banner")
        if scale:
            return scale.url
        return ""

    def getHeader(self):
        if self.prologue:
            text = self.prologue.output
        else:
            text = ""
        return text

    def getFooter(self):
        if self.epilogue:
            text = self.epilogue.output
        else:
            text = ""
        return text

    def getText(self):
        if self.text:
            text = self.text.output
        else:
            text = ""
        return text

    # bbb: we should print a deprecation message here
    def getOutputTemplate(self):
        return self.output_template


class ISendStatus(Interface):
    """Manage send status for newsletter issues."""

    def clear():  # noqa: N805
        """Clear all records."""

    def add_records(records):  # noqa: N805
        """Add new records."""

    def get_records(status=None):  # noqa: N805
        """Return a list of all status information records."""

    def get_keys(status=None):  # noqa: N805
        """Return a list of all status information keys."""


@implementer(ISendStatus)
class SendStatus:
    def __init__(self, context):
        self.context = context

    def clear(self):
        """Clear all records."""
        annotations = IAnnotations(self.context)
        if SEND_STATUS_KEY in annotations:
            del annotations[SEND_STATUS_KEY]

    def add_records(self, records, key="email"):
        """Add new records."""
        annotations = IAnnotations(self.context)
        if SEND_STATUS_KEY not in annotations:
            annotations[SEND_STATUS_KEY] = PersistentDict()
        data = {data.get(key): data for data in records if data.get(key) is not None}
        annotations[SEND_STATUS_KEY].update(data)

    def get_records(self, successful=None):
        """Return a list of all status information records."""
        annotations = IAnnotations(self.context)
        if SEND_STATUS_KEY not in annotations:
            return []
        items = annotations[SEND_STATUS_KEY].values()
        if successful is not None:
            items = [
                item
                for item in items
                if item.get("status", {}).get("successful", None) == successful
            ]
        return items

    def get_keys(self, successful=None):
        """Return a list of all status information keys."""
        annotations = IAnnotations(self.context)
        if SEND_STATUS_KEY not in annotations:
            return []
        items = annotations[SEND_STATUS_KEY].items()
        if successful is not None:
            items = [
                key
                for key, item in items
                if item.get("status", {}).get("successful", None) == successful
            ]
        return items



import pkg_resources
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import (
    PLONE_FIXTURE,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
    applyProfile,
)
from plone.testing import z2

from Products.EasyNewsletter.tests.base import enable_behavior

try:
    pkg_resources.get_distribution("plone.app.contenttypes")

except pkg_resources.DistributionNotFound:
    HAS_PACT = False
else:
    HAS_PACT = True


class EasyNewsletter(PloneSandboxLayer):
    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        if HAS_PACT:
            z2.installProduct(app, "plone.app.contenttypes")
        # Load ZCML
        import Products.EasyNewsletter

        self.loadZCML(package=Products.EasyNewsletter)

        # Install product and call its initialize() function
        z2.installProduct(app, "Products.EasyNewsletter")

    def setUpPloneSite(self, portal):
        if HAS_PACT:
            applyProfile(portal, "plone.app.contenttypes:default")
        # Install into Plone site using portal_setup
        applyProfile(portal, "Products.EasyNewsletter:default")
        # applyProfile(portal, 'Products.EasyNewsletter:install-base')
        enable_behavior("Newsletter", "Products.EasyNewsletter.plone_user_group_recipients")
        enable_behavior("Newsletter Issue", "Products.EasyNewsletter.plone_user_group_recipients")

    def tearDownZope(self, app):
        # Uninstall product
        z2.uninstallProduct(app, "Products.EasyNewsletter")


EASYNEWSLETTER_FIXTURE = EasyNewsletter()

PRODUCTS_EASYNEWSLETTER_INTEGRATION_TESTING = IntegrationTesting(
    bases=(EASYNEWSLETTER_FIXTURE,), name="EasyNewsletter:Integration"
)

PRODUCTS_EASYNEWSLETTER_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(EASYNEWSLETTER_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="EasyNewsletter:Functional",
)

PRODUCTS_EASYNEWSLETTER_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(EASYNEWSLETTER_FIXTURE, REMOTE_LIBRARY_BUNDLE_FIXTURE, z2.ZSERVER_FIXTURE),
    name="EasyNewsletter:Acceptance",
)
